@@ -1,20 +1,6 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Assignment related nodes.
 
 The most simple assignment statement ``a = b`` is what we have here. All others
@@ -45,7 +31,8 @@ from .shapes.ControlFlowDescriptions import (
     ControlFlowDescriptionFullEscape,
     ControlFlowDescriptionNoEscape,
 )
-from .shapes.StandardShapes import tshape_iterator, tshape_unknown
+from .shapes.IteratorShapes import tshape_iterator
+from .shapes.StandardShapes import tshape_unknown
 from .StatementBasesGenerated import (
     StatementAssignmentVariableConstantImmutableBase,
     StatementAssignmentVariableConstantMutableBase,
@@ -179,19 +166,7 @@ class StatementAssignmentVariableMixin(object):
         # is such a code for the type shape.
         return True
 
-    def _transferState(self, result):
-        self.variable_trace.assign_node = result
-        result.variable_trace = self.variable_trace
-        self.variable_trace = None
-
-    def _considerSpecialization(self, old_source):
-        # Specialize if possible, might have become that way only recently.
-        # return driven, pylint: disable=too-many-return-statements
-        source = self.subnode_source
-
-        if source is old_source:
-            return self, None, None
-
+    def _considerSpecialization(self, old_source, source):
         if source.isCompileTimeConstant():
             result = makeStatementAssignmentVariableConstant(
                 source=source,
@@ -201,8 +176,6 @@ class StatementAssignmentVariableMixin(object):
                 source_ref=self.source_ref,
             )
 
-            self._transferState(result)
-
             return (
                 result,
                 "new_statements",
@@ -218,8 +191,6 @@ class StatementAssignmentVariableMixin(object):
                 source_ref=self.source_ref,
             )
 
-            self._transferState(result)
-
             return (
                 result,
                 "new_statements",
@@ -234,8 +205,6 @@ class StatementAssignmentVariableMixin(object):
                 source_ref=self.source_ref,
             )
 
-            self._transferState(result)
-
             return (
                 result,
                 "new_statements",
@@ -250,8 +219,6 @@ class StatementAssignmentVariableMixin(object):
                 source_ref=self.source_ref,
             )
 
-            self._transferState(result)
-
             return (
                 result,
                 "new_statements",
@@ -266,8 +233,6 @@ class StatementAssignmentVariableMixin(object):
                 source_ref=self.source_ref,
             )
 
-            self._transferState(result)
-
             return (
                 result,
                 "new_statements",
@@ -284,6 +249,10 @@ class StatementAssignmentVariableMixin(object):
     def hasVeryTrustedValue(self):
         """Does this assignment node have a very trusted value."""
 
+    @abstractmethod
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
+        """Set the value trace for the assignment form."""
+
 
 class StatementAssignmentVariableGeneric(
     StatementAssignmentVariableMixin, StatementAssignmentVariableGenericBase
@@ -301,7 +270,7 @@ class StatementAssignmentVariableGeneric(
         "inplace_suspect",
     )
 
-    # false alarm due to post_init, pylint: disable=attribute-defined-outside-init
+    # False alarm due to post_init, pylint: disable=attribute-defined-outside-init
 
     # TODO: Add parsing of node_attributes for init values, so we can avoid these too
     def postInitNode(self):
@@ -379,7 +348,6 @@ Assignment raises exception in assigned value, removed assignment.""",
             and source.isExpressionVariableRef()
             and source.getVariable() is variable
         ):
-
             # A variable access that has a side effect, must be preserved,
             # so it can e.g. raise an exception, otherwise we can be fully
             # removed.
@@ -404,17 +372,26 @@ Removed assignment of %s from itself which is known to be defined."""
                     % variable.getDescription(),
                 )
 
+        if source is old_source:
+            result = self, None, None
+        else:
+            result = self._considerSpecialization(old_source, source)
+            result[0].parent = self.parent
+
+        result[0].computeStatementAssignmentTraceUpdate(trace_collection, source)
+
+        return result
+
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
         # Set-up the trace to the trace collection, so future references will
         # find this assignment.
         self.variable_trace = trace_collection.onVariableSet(
-            variable=variable, version=self.variable_version, assign_node=self
+            variable=self.variable, version=self.variable_version, assign_node=self
         )
 
         # TODO: Determine from future use of assigned variable, if this is needed at all.
         trace_collection.removeKnowledge(source)
 
-        return self._considerSpecialization(old_source)
-
     def hasVeryTrustedValue(self):
         """Does this assignment node have a very trusted value."""
         return self.subnode_source.hasVeryTrustedValue()
@@ -447,7 +424,7 @@ class StatementAssignmentVariableIterator(
         "is_indexable",
     )
 
-    # false alarm due to post_init, pylint: disable=attribute-defined-outside-init
+    # False alarm due to post_init, pylint: disable=attribute-defined-outside-init
 
     # TODO: Add parsing of node_attributes for init values, so we can avoid these too
     def postInitNode(self):
@@ -487,7 +464,7 @@ class StatementAssignmentVariableIterator(
         self.temp_scope = provider.allocateTempScope("iterator_access")
 
         self.tmp_iterated_variable = provider.allocateTempVariable(
-            temp_scope=self.temp_scope, name="iterated_value"
+            temp_scope=self.temp_scope, name="iterated_value", temp_type="object"
         )
 
         reference_iterated = ExpressionTempVariableRef(
@@ -500,18 +477,16 @@ class StatementAssignmentVariableIterator(
         assign_iterated = makeStatementAssignmentVariable(
             source=iterated_value,
             variable=self.tmp_iterated_variable,
-            variable_version=None,
             source_ref=iterated_value.source_ref,
         )
 
         self.tmp_iteration_count_variable = provider.allocateTempVariable(
-            temp_scope=self.temp_scope, name="iteration_count"
+            temp_scope=self.temp_scope, name="iteration_count", temp_type="object"
         )
 
         assign_iteration_count = makeStatementAssignmentVariable(
             source=makeConstantRefNode(constant=0, source_ref=self.source_ref),
             variable=self.tmp_iteration_count_variable,
-            variable_version=None,
             source_ref=iterated_value.source_ref,
         )
 
@@ -531,7 +506,7 @@ class StatementAssignmentVariableIterator(
 
         # For use when the "next" is replaced.
         self.tmp_iteration_next_variable = provider.allocateTempVariable(
-            temp_scope=self.temp_scope, name="next_value"
+            temp_scope=self.temp_scope, name="next_value", temp_type="object"
         )
 
         result = makeStatementsSequenceReplacementNode(
@@ -596,12 +571,18 @@ Assignment raises exception in assigned value, removed assignment.""",
 
         # Set-up the trace to the trace collection, so future references will
         # find this assignment.
+        # Note: Keep this aligned with computeStatementAssignmentTraceUpdate
         self.variable_trace = trace_collection.onVariableSet(
             variable=variable, version=self.variable_version, assign_node=self
         )
 
         return self, None, None
 
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
+        self.variable_trace = trace_collection.onVariableSet(
+            variable=self.variable, version=self.variable_version, assign_node=self
+        )
+
     @staticmethod
     def hasVeryTrustedValue():
         """Does this assignment node have a very trusted value."""
@@ -624,7 +605,7 @@ class StatementAssignmentVariableConstantMutable(
         "inplace_suspect",
     )
 
-    # false alarm due to post_init, pylint: disable=attribute-defined-outside-init
+    # False alarm due to post_init, pylint: disable=attribute-defined-outside-init
 
     # TODO: Add parsing of node_attributes for init values, so we can avoid these too
     def postInitNode(self):
@@ -644,6 +625,7 @@ class StatementAssignmentVariableConstantMutable(
 
         # Set-up the trace to the trace collection, so future references will
         # find this assignment.
+        # Note: Keep this aligned with computeStatementAssignmentTraceUpdate
         self.variable_trace = trace_collection.onVariableSet(
             variable=variable, version=self.variable_version, assign_node=self
         )
@@ -684,6 +666,11 @@ class StatementAssignmentVariableConstantMutable(
 
         return self, None, None
 
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
+        self.variable_trace = trace_collection.onVariableSet(
+            variable=self.variable, version=self.variable_version, assign_node=self
+        )
+
     @staticmethod
     def hasVeryTrustedValue():
         """Does this assignment node have a very trusted value."""
@@ -706,7 +693,7 @@ class StatementAssignmentVariableConstantImmutable(
         "inplace_suspect",
     )
 
-    # false alarm due to post_init, pylint: disable=attribute-defined-outside-init
+    # False alarm due to post_init, pylint: disable=attribute-defined-outside-init
 
     # TODO: Add parsing of node_attributes for init values, so we can avoid these too
     def postInitNode(self):
@@ -787,12 +774,18 @@ class StatementAssignmentVariableConstantImmutable(
                         % self.getVariableName(),
                     )
 
+        # Note: Keep this aligned with computeStatementAssignmentTraceUpdate
         self.variable_trace = trace_collection.onVariableSetToUnescapableValue(
             variable=variable, version=self.variable_version, assign_node=self
         )
 
         return self, None, None
 
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
+        self.variable_trace = trace_collection.onVariableSetToUnescapableValue(
+            variable=self.variable, version=self.variable_version, assign_node=self
+        )
+
     @staticmethod
     def hasVeryTrustedValue():
         """Does this assignment node have a very trusted value."""
@@ -806,7 +799,7 @@ class StatementAssignmentVariableConstantMutableTrusted(
 
     @staticmethod
     def hasVeryTrustedValue():
-        return True
+        return False
 
 
 class StatementAssignmentVariableConstantImmutableTrusted(
@@ -835,7 +828,7 @@ class StatementAssignmentVariableHardValue(
         "inplace_suspect",
     )
 
-    # false alarm due to post_init, pylint: disable=attribute-defined-outside-init
+    # False alarm due to post_init, pylint: disable=attribute-defined-outside-init
 
     # TODO: Add parsing of node_attributes for init values, so we can avoid these too
     def postInitNode(self):
@@ -870,12 +863,18 @@ class StatementAssignmentVariableHardValue(
 Assignment raises exception in assigned value, removed assignment.""",
             )
 
+        # Note: Keep this aligned with computeStatementAssignmentTraceUpdate
         self.variable_trace = trace_collection.onVariableSetToVeryTrustedValue(
             variable=variable, version=self.variable_version, assign_node=self
         )
 
         return self, None, None
 
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
+        self.variable_trace = trace_collection.onVariableSetToVeryTrustedValue(
+            variable=self.variable, version=self.variable_version, assign_node=self
+        )
+
     @staticmethod
     def hasVeryTrustedValue():
         """Does this assignment node have a very trusted value."""
@@ -898,7 +897,7 @@ class StatementAssignmentVariableFromVariable(
         "inplace_suspect",
     )
 
-    # false alarm due to post_init, pylint: disable=attribute-defined-outside-init
+    # False alarm due to post_init, pylint: disable=attribute-defined-outside-init
 
     # TODO: Add parsing of node_attributes for init values, so we can avoid these too
     def postInitNode(self):
@@ -916,6 +915,11 @@ class StatementAssignmentVariableFromVariable(
         old_source = self.subnode_source
         variable = self.variable
 
+        # Assigning from and to the same variable, can be optimized away
+        # immediately, there is no point in doing it. Exceptions are of course
+        # module variables that collide with built-in names. TODO: In
+        # specialization this could be considered right away as its own node
+        # type, waiting for it to compute like this.
         if not variable.isModuleVariable() and old_source.getVariable() is variable:
             # A variable access that has a side effect, must be preserved,
             # so it can e.g. raise an exception, otherwise we can be fully
@@ -925,8 +929,10 @@ class StatementAssignmentVariableFromVariable(
                     expression=old_source, node=self
                 )
 
+                result = trace_collection.onStatement(result)
+
                 return (
-                    result.computeStatementsSequence(trace_collection),
+                    result,
                     "new_statements",
                     """\
 Lowered assignment of %s from itself to mere access of it."""
@@ -944,20 +950,47 @@ Removed assignment of %s from itself which is known to be defined."""
         # Let assignment source may re-compute first.
         source = trace_collection.onExpression(self.subnode_source)
 
-        # Assigning from and to the same variable, can be optimized away
-        # immediately, there is no point in doing it. Exceptions are of course
-        # module variables that collide with built-in names.
+        if source is old_source:
+            result = self, None, None
+        else:
+            if source.willRaiseAnyException():
+                result = makeStatementExpressionOnlyReplacementNode(
+                    expression=source, node=self
+                )
 
-        # Set-up the trace to the trace collection, so future references will
-        # find this assignment.
-        self.variable_trace = trace_collection.onVariableSet(
-            variable=variable, version=self.variable_version, assign_node=self
-        )
+                return (
+                    result,
+                    "new_raise",
+                    """\
+Assignment raises exception in assigned variable access, removed assignment.""",
+                )
 
-        # TODO: Determine from future use of assigned variable, if this is needed at all.
-        trace_collection.removeKnowledge(source)
+            result = self._considerSpecialization(old_source, source)
+            result[0].parent = self.parent
 
-        return self._considerSpecialization(old_source)
+        result[0].computeStatementAssignmentTraceUpdate(trace_collection, source)
+
+        return result
+
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
+        if source.isExpressionVariableRef():
+            self.variable_trace = trace_collection.onVariableSetAliasing(
+                variable=self.variable,
+                version=self.variable_version,
+                assign_node=self,
+                source=source,
+            )
+        else:
+            # Set-up the trace to the trace collection, so future references will
+            # find this assignment. TODO: We should for non-variables make sure we do
+            # always specialize, since this is no longer a variable once it was
+            # resolved.
+            self.variable_trace = trace_collection.onVariableSet(
+                variable=self.variable, version=self.variable_version, assign_node=self
+            )
+
+            # TODO: Determine from future use of assigned variable, if this is needed at all.
+            trace_collection.removeKnowledge(source)
 
     @staticmethod
     def hasVeryTrustedValue():
@@ -981,7 +1014,7 @@ class StatementAssignmentVariableFromTempVariable(
         "inplace_suspect",
     )
 
-    # false alarm due to post_init, pylint: disable=attribute-defined-outside-init
+    # False alarm due to post_init, pylint: disable=attribute-defined-outside-init
 
     # TODO: Add parsing of node_attributes for init values, so we can avoid these too
     def postInitNode(self):
@@ -1014,17 +1047,26 @@ Removed assignment of %s from itself which is known to be defined."""
         # Let assignment source may re-compute first.
         source = trace_collection.onExpression(self.subnode_source)
 
+        if source is old_source:
+            result = self, None, None
+        else:
+            result = self._considerSpecialization(old_source, source)
+            result[0].parent = self.parent
+
+        result[0].computeStatementAssignmentTraceUpdate(trace_collection, source)
+
+        return result
+
+    def computeStatementAssignmentTraceUpdate(self, trace_collection, source):
         # Set-up the trace to the trace collection, so future references will
         # find this assignment.
         self.variable_trace = trace_collection.onVariableSet(
-            variable=variable, version=self.variable_version, assign_node=self
+            variable=self.variable, version=self.variable_version, assign_node=self
         )
 
         # TODO: Determine from future use of assigned variable, if this is needed at all.
         trace_collection.removeKnowledge(source)
 
-        return self._considerSpecialization(old_source)
-
     @staticmethod
     def hasVeryTrustedValue():
         """Does this assignment node have a very trusted value."""
@@ -1117,3 +1159,19 @@ def makeStatementAssignmentVariable(
             variable_version=variable_version,
             source_ref=source_ref,
         )
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
