@@ -1,4 +1,4 @@
-#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
 """ Creating virtualenvs and running commands in them.
@@ -10,7 +10,6 @@ import sys
 from contextlib import contextmanager
 
 from nuitka.__past__ import unicode
-from nuitka.Tracing import my_print
 from nuitka.utils.Execution import (
     NuitkaCalledProcessError,
     check_call,
@@ -19,34 +18,45 @@ from nuitka.utils.Execution import (
 )
 from nuitka.utils.FileOperations import (
     getDirectoryRealPath,
+    relpath,
     removeDirectory,
     withDirectoryChange,
 )
 
 
 class Virtualenv(object):
-    def __init__(self, env_dir):
+    def __init__(self, env_dir, logger):
         self.env_dir = os.path.abspath(env_dir)
+        self.logger = logger
 
-    def runCommand(self, commands, env=None, style=None):
+    def runCommand(self, commands, keep_cwd=False, env=None, style=None):
         if type(commands) in (str, unicode):
             commands = [commands]
 
-        with withDirectoryChange(self.env_dir):
-            if os.name == "nt":
-                commands = [r"call scripts\activate.bat"] + commands
+        with withDirectoryChange(None if keep_cwd else self.env_dir, allow_none=True):
+            if keep_cwd:
+                activate_dir = relpath(self.env_dir, os.getcwd())
             else:
-                commands = [". bin/activate"] + commands
+                activate_dir = "."
+
+            if os.name == "nt":
+                commands = [r"call %s\scripts\activate.bat" % activate_dir] + commands
+            else:
+                commands = [". %s/bin/activate" % activate_dir] + commands
 
             command = " && ".join(commands)
 
             if style is not None:
-                my_print("Executing: %s" % command, style=style)
+                self.logger.info("Executing: %s" % command, style=style)
 
             with withEnvironmentVarsOverridden(env):
                 exit_code = os.system(command)
                 if exit_code != 0:
-                    my_print("Failure %s for: %s" % (exit_code, command), style=style)
+                    self.logger.info(
+                        "Failure %s for: %s" % (exit_code, command),
+                        keep_format=True,
+                        style=style,
+                    )
 
                     raise NuitkaCalledProcessError(
                         exit_code, command, output=None, stderr=None
@@ -69,7 +79,7 @@ class Virtualenv(object):
             command = " && ".join(commands)
 
             if style is not None:
-                my_print("Executing: %s" % command, style=style)
+                self.logger.info("Executing: %s" % command, style=style)
 
             # Use subprocess and also return outputs, stdout, stderr, result
             return executeProcess(
@@ -82,14 +92,16 @@ class Virtualenv(object):
 
 
 @contextmanager
-def withVirtualenv(env_name, base_dir=None, python=None, delete=True, style=None):
+def withVirtualenv(
+    env_name, logger, base_dir=None, python=None, delete=True, style=None
+):
     """Create a virtualenv and change into it.
 
     Activating for actual use will be your task.
     """
 
     if style is not None:
-        my_print("Creating a virtualenv:")
+        logger.info("Creating a virtualenv:")
 
     if python is None:
         python = sys.executable
@@ -105,19 +117,29 @@ def withVirtualenv(env_name, base_dir=None, python=None, delete=True, style=None
     else:
         env_dir = env_name
 
-    removeDirectory(env_dir, ignore_errors=False)
+    removeDirectory(
+        env_dir,
+        logger=logger,
+        ignore_errors=False,
+        extra_recommendation=None,
+    )
 
     with withDirectoryChange(base_dir, allow_none=True):
         command = [python, "-m", "virtualenv", env_name]
         if style is not None:
-            my_print("Executing: %s" % " ".join(command), style=style)
+            logger.info("Executing: %s" % " ".join(command), style=style)
         check_call(command)
 
     try:
-        yield Virtualenv(env_dir)
+        yield Virtualenv(env_dir, logger=logger)
     finally:
         if delete:
-            removeDirectory(env_dir, ignore_errors=False)
+            removeDirectory(
+                env_dir,
+                logger=logger,
+                ignore_errors=False,
+                extra_recommendation=None,
+            )
 
 
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
